//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This shows how to make use of joypads within your 
///			glut programs. The included files Joypad.h and Joypad.cpp
///			forms a wrapper over DirectX Input and the linux joystick
///			lib. 
///				This code should compile on both platforms however
///			you will need the DirectX SDK installed under Windows,
///			and the linux joystick lib installed under linux.
//------------------------------------------------------------

#include <stdlib.h>
#include <math.h>
#include <GL/glut.h>

#include "Joypad.h"

#define BSIZE 40 
#define ASIZE 200 
#define RADIUS 50.0f 

/// interface to the joypad
XInputJoypad* g_pJoypad=0;

//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	if (h==0) {
		h=1;
	}

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	glOrtho(0,w,h,0,-100,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------	DrawButton()
//
void DrawButton(int x,int y,bool state) {

	// draw filled or wireframe depending on state
	if (state) 
		glBegin(GL_QUADS);
	else
		glBegin(GL_LINE_LOOP);

	glVertex2i(x,y);
	glVertex2i(x,y+BSIZE);
	glVertex2i(x+BSIZE,y+BSIZE);
	glVertex2i(x+BSIZE,y);

	glEnd();
}

//------------------------------------------------------------	DrawAxis()
//
void DrawAxis(int x,int y,long value) {

	// put value into 0 to 1 range rather than -32766 to +32766
	float fval = (value/32766.0f+1.0f)*0.5f;

	// draw box
	glBegin(GL_LINE_LOOP);

		glVertex2f(x,y);
		glVertex2f(x,y+ASIZE);
		glVertex2f(x+BSIZE,y+ASIZE);
		glVertex2f(x+BSIZE,y);

	glEnd();

	// scale value to the size of the axis display
	fval *= ASIZE;

	// draw line to represent joypad position
	glBegin(GL_LINES);
		glVertex2f(x,y+fval);
		glVertex2f(x+BSIZE,y+fval);
	glEnd();
}

//------------------------------------------------------------	DrawPOV()
//
void DrawPOV(int x,int y,long angle) {
	float f = 0.0f;

	// draw a circle
	glBegin(GL_LINE_LOOP);
	for( ; f <6.2823f;f+=0.3f) 
		glVertex2f(x+RADIUS*cos(f),y+RADIUS*sin(f));
	glEnd();

	// draw direction of POV
	float radians = (angle/100.0f) * 3.14159f / 180.0f;

	glBegin(GL_LINES);
		glVertex2f(x,y);
		glVertex2f(x+RADIUS*sin(radians),y+RADIUS*(-cos(radians)));
	glEnd();
    
}


//------------------------------------------------------------	OnDraw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// loop through each button on the joypad and display 
	// it's state.
	// 
	for(unsigned int i=0;i!=g_pJoypad->NumButtons();++i) {
		DrawButton(i*(BSIZE+10)+10,10,g_pJoypad->Button(i));
	}

	// loop through each axis control on the joypad and display
	// it's current value
	//
	int num=0;
	if (g_pJoypad->HasX()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Xaxis());
		num++;
	}
	if (g_pJoypad->HasY()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Yaxis());
		num++;
	}
	if (g_pJoypad->HasZ()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Zaxis());
		num++;
	}

	if (g_pJoypad->HasXrot()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Xrot());
		num++;
	}
	if (g_pJoypad->HasYrot()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Yrot());
		num++;
	}
	if (g_pJoypad->HasZrot()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->Zrot());
		num++;
	}

	if (g_pJoypad->HasExtra1()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->ExtraAxes(0));
		num++;
	}
	if (g_pJoypad->HasExtra2()) {
		DrawAxis(num*(BSIZE+10)+10,60,g_pJoypad->ExtraAxes(1));
		num++;
	}

	// loop through each point of view control and display it's direction value
	for(unsigned int i=0;i!=g_pJoypad->NumPovs();++i) {
		DrawPOV(i*2*(RADIUS+10)+10+RADIUS,270+RADIUS,g_pJoypad->POV(i));
	}

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}


//------------------------------------------------------------	OnInit()
//
void OnInit() {

	// first initialise Direct Input or the linux joystick lib
	if (!InitInput()) {
		std::cerr << "Failed to initialise the joypad lib\n";
		exit(0);
	}

	// make sure there is a joypad attached
	if (NumJoypads()==0) {
		std::cerr << "No Joypad attached\n";
		exit(0);
	}

	// get the first available jopypad
	if( !(g_pJoypad = GetJoypad(0)) ) {
		std::cerr << "Failed to access joypad\n";
		exit(0);
	}
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {

	// make sure we clean up directX and the linux joystick lib
	FreeInput();
}

//------------------------------------------------------------	OnIdle()
//
void OnIdle() {

	// update all connected joypads
	UpdateInputState();

	// redraw the screen
	glutPostRedisplay();
}


//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE);

	// set the initial window size
	glutInitWindowSize(640,480);

	// create the window
	glutCreateWindow("Using Joypads in glut");

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// set the idle callback
	glutIdleFunc(OnIdle);
	
	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
